module modul_studenta (
    input  logic        rst,
    input  logic        clk,

    output logic        s_axil_awready,
    input  wire         s_axil_awvalid,
    input  wire [20:0]  s_axil_awaddr,
    input  wire [2:0]   s_axil_awprot,
    output logic        s_axil_wready,
    input  wire         s_axil_wvalid,
    input  wire [31:0]  s_axil_wdata,
    input  wire [3:0]   s_axil_wstrb,
    input  wire         s_axil_bready,
    output logic        s_axil_bvalid,
    output logic [1:0]  s_axil_bresp,
    output logic        s_axil_arready,
    input  wire         s_axil_arvalid,
    input  wire [20:0]  s_axil_araddr,
    input  wire [2:0]   s_axil_arprot,
    input  wire         s_axil_rready,
    output logic        s_axil_rvalid,
    output logic [31:0] s_axil_rdata,
    output logic [1:0]  s_axil_rresp,

    output logic [7:0]  LED
);

import registers_pkg::*;

registers_pkg::registers__out_t hwif_out;

//====================================================
//=======  FSM Parameters  ===========================
//====================================================
parameter       SIZE = 4      ;

parameter       IDLE  = 4'b0000,
                LED1  = 4'b0001,
                LED2  = 4'b0010,
                LED3  = 4'b0011,
                LED4  = 4'b0100,
                LED5  = 4'b0101,
                LED6  = 4'b0110,
                LED7  = 4'b0111,
                LED8  = 4'b1000;


logic   [SIZE-1:0]          state        ;// Seq part of the FSM
logic   [SIZE-1:0]          next_state   ;// combo part of FSM
logic   [7:0]               LEDLogic     ;
logic                       FSM_enable   ;
logic   [31:0]              COUNTER_VALUE;

//assign LED[0] = hwif_out.LEDS.LED0.value;
//assign LED[7] = 1'b1;

assign LED = LEDLogic;
assign COUNTER_VALUE = hwif_out.COUNTER_VALUE.VALUE.value;
assign FSM_enable = hwif_out.FSM.ENABLE.value ;

//------------------------------------------
//------------- Registers ------------------
//------------------------------------------
registers u_registers (
    .clk                (clk),
    .rst                (rst),

    .s_axil_awready     (s_axil_awready),
    .s_axil_awvalid     (s_axil_awvalid),
    .s_axil_awaddr      (s_axil_awaddr[3:0]),
    .s_axil_awprot      (s_axil_awprot),

    .s_axil_wready      (s_axil_wready),
    .s_axil_wvalid      (s_axil_wvalid),
    .s_axil_wdata       (s_axil_wdata),
    .s_axil_wstrb       (s_axil_wstrb),

    .s_axil_bready      (s_axil_bready),
    .s_axil_bvalid      (s_axil_bvalid),
    .s_axil_bresp       (s_axil_bresp),

    .s_axil_arready     (s_axil_arready),
    .s_axil_arvalid     (s_axil_arvalid),
    .s_axil_araddr      (s_axil_araddr[3:0]),
    .s_axil_arprot      (s_axil_arprot),

    .s_axil_rready      (s_axil_rready),
    .s_axil_rvalid      (s_axil_rvalid),
    .s_axil_rdata       (s_axil_rdata),
    .s_axil_rresp       (s_axil_rresp),

    .hwif_out           (hwif_out)
);


//====================================================
//=======  COUNTER  ==================================
//====================================================
logic [31:0] fsm_counter;

always @ (posedge clk)
 begin
   if (rst == 1'b1) begin
     fsm_counter <=  '0;
   end else if (fsm_counter != COUNTER_VALUE && fsm_counter <= COUNTER_VALUE) begin
     fsm_counter <=  fsm_counter + 1'b1;
   end else begin
     fsm_counter <= '0;
   end
 end


//====================================================
//=======  PULSE REG  ================================
//====================================================
//logic fsm_counter_reg;
logic fsm_counter_pulse;

//always @ (posedge clk)
// begin
//   if (rst == 1'b1) begin
//     fsm_counter_reg <=  '0;
//   end else begin
//     fsm_counter_reg <=  fsm_counter[23];
//   end
// end

 assign fsm_counter_pulse = fsm_counter == COUNTER_VALUE;


//====================================================
//=======  FSM STATE REGISTER  =======================
//====================================================
always @ (posedge clk)
 begin : FSM_SEQ
   if (rst == 1'b1) begin
        state <=  IDLE;
   end else begin
        state <=  next_state;
   end
 end


//====================================================
//=======  FSM STATE LOGIC  ==========================
//====================================================
always @ (*)
 begin : FSM_COMBO
  
  next_state = IDLE;
  
  case(state)
    IDLE :  if (FSM_enable != 1'b1) begin
                next_state = IDLE;
            end
            else if (fsm_counter_pulse == 1'b1) begin
                next_state = LED1;
            end else begin
                next_state = IDLE;
            end

    LED1 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED2;
            end else begin
                next_state = LED1;
            end

    LED2 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED3;
            end else begin
                next_state = LED2;
            end

    LED3 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED4;
            end else begin
                next_state = LED3;
            end
    
    LED4 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED5;
            end else begin
                next_state = LED4;
            end

    LED5 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED6;
            end else begin
                next_state = LED5;
            end

    LED6 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED7;
            end else begin
                next_state = LED6;
            end

    LED7 :  if (fsm_counter_pulse == 1'b1) begin
                next_state = LED8;
            end else begin
                next_state = LED7;
            end

    LED8 :  if (FSM_enable != 1'b1) begin
                next_state = IDLE;
            end 
            else if (fsm_counter_pulse == 1'b1) begin
                next_state = LED1;
            end else begin
                next_state = LED8;
            end

    default : next_state = IDLE;
   endcase
 end


//====================================================
//=======  FSM OUTPUT LOGIC  ==========================
//====================================================
always @ (*)
begin : OUTPUT_LOGIC
  case(state)
   IDLE : begin
           LEDLogic =  8'b0000000000;
          end
   LED1 : begin
           LEDLogic =  8'b0000000001;
          end
   LED2 : begin
           LEDLogic =  8'b0000000010;
          end
   LED3 : begin
           LEDLogic =  8'b0000000100;
          end
   LED4 : begin
           LEDLogic =  8'b0000001000;
          end
   LED5 : begin
           LEDLogic =  8'b0000010000;
          end
   LED6 : begin
           LEDLogic =  8'b0000100000;
          end
   LED7 : begin
           LEDLogic =  8'b0001000000;
          end
   LED8 : begin
           LEDLogic =  8'b0010000000;
          end
   default : begin
           LEDLogic =  8'b0000000000;
             end
  endcase
end


`ifdef FSM_DEBUG

(* noprune *) logic [SIZE-1:0]  DEBUG_state        ;// Seq part of the FSM
(* noprune *) logic [SIZE-1:0]  DEBUG_next_state   ;// combo part of FSM
(* noprune *) logic [7:0]       DEBUG_LEDLogic     ;
(* noprune *) logic             DEBUG_FSM_enable   ;
(* noprune *) logic [31:0]      DEBUG_fsm_counter  ;


always_ff @(posedge clk) begin

    DEBUG_state       <= state       ;
    DEBUG_next_state  <= next_state  ;
    DEBUG_LEDLogic    <= LEDLogic    ;
    DEBUG_FSM_enable  <= FSM_enable  ;
    DEBUG_fsm_counter <= fsm_counter ;
end
`endif






endmodule